VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDeltaT"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use


Function DeltaT(dblFixed As Double, Optional blnFromUTC As Boolean = True) As Double
'Find the difference between UTC1 and TD
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 77
'according to
'J. Chapront, M. Chapront-Touze and G. Francou, Note s055, bureau des Longitudes, Paris, Dec. 1997
'dblfixed is fixed radix from 1 jan 1 0:00 UTC gregorian = 1
    Dim dblJYear As Double
    Dim dblB1 As Double
    Dim dblB2 As Double
    Dim strLine As String
    Dim intI As Integer
    Dim dblY(3) As Double
    Dim dblT As Double
    Dim dblCorrection As Double 'Correction for continuity
    Dim blnFinished As Boolean
    Dim dblOldDeltaT As Double
    Dim intN As Double
    Dim varTemp As Variant
    
    If Not blnDeltaTRead Then 'First time, read table (biannual 1620 to 2000)
        Open "DeltaT.txt" For Input Access Read As 1
        For intI = 0 To NDeltaTModern - 1
            varTemp = GetValue(1, "#", "Double", -10, 130)
            If IsNull(varTemp) Then
                MsgBox "Error in DeltaT file"
                End
            End If
            dblT = intI / 50 + 16.2
            dblDeltaT(intI) = varTemp
            'If preatomic time then adjust for lunar acceleration
            If dblT < 19.555 Then dblDeltaT(intI) = dblDeltaT(intI) - 0.91072 * (LunAcc - LunAccDTTables) * Square(dblT - 19.555)
        Next
        Close 1
        Open "PastDeltaT.txt" For Input Access Read As 1
        For intI = 0 To 42
            varTemp = GetValue(1, "#", "Double", 100, 17000)
            If IsNull(varTemp) Then
                MsgBox "Error in DeltaT file"
                End
            End If
            dblT = intI / 2 - 5
            dblPastDeltaT(intI) = varTemp
            'Adjust for lunar acceleration
            dblPastDeltaT(intI) = dblPastDeltaT(intI) - 0.91072 * (LunAcc - LunAccDTTables) * Square(dblT - 19.555)
        Next
        Close 1
        blnDeltaTRead = True
    End If
    DeltaT = 0
    dblOldDeltaT = 0
    blnFinished = False
    While Not blnFinished
        dblJYear = (dblFixed + 14.25 - DeltaT / 86400 + 365.25) / 365.25 'Julian year
        dblT = (dblJYear - 2000) / 100 'Julian centuries from 2000
        If dblJYear < -500 Then 'Distant past use JPL formula and correct for difference in lunar acceleration
            DeltaT = 31 * Square((dblJYear - 1820) / 100) - 0.91072 * (LunAcc - LunAccPast) * Square((dblJYear - 1955.5) / 100)
            If dblJYear > -600 Then 'Interpolate to the beginning of the first table
                dblCorrection = 31 * Square((-500 - 1820) / 100) - 0.91072 * (LunAcc - LunAccPast) * Square((dblJYear - 1955.5) / 100) - dblPastDeltaT(0)
                DeltaT = DeltaT - (dblJYear + 600) * dblCorrection / 100
            End If
        ElseIf dblJYear < -475 Then 'Linear interpolation of 1st two points of first table
            dblY(0) = dblPastDeltaT(0)
            dblY(1) = dblPastDeltaT(1)
            DeltaT = Interp(dblY, (dblJYear + 500) / 50, 2)
        ElseIf dblJYear < 1575 Then 'Three point interpolation of first table
            dblY(0) = dblPastDeltaT((dblJYear + 475) \ 50)
            dblY(1) = dblPastDeltaT((dblJYear + 475) \ 50 + 1)
            dblY(2) = dblPastDeltaT((dblJYear + 475) \ 50 + 2)
            DeltaT = Interp(dblY, dblJYear / 50 - Int((dblJYear - 25) / 50), 3)
        ElseIf dblJYear < 1600 Then 'Linear interpolation of last two points
            dblY(0) = dblPastDeltaT(41)
            dblY(1) = dblPastDeltaT(42)
            DeltaT = Interp(dblY, (dblJYear - 1550) / 50, 2)
        ElseIf dblJYear > 1618 + NDeltaTModern * 2 Then 'Future
            DeltaT = 102 + dblT * (102 + dblT * 25.3)
            If dblJYear < 1718 + NDeltaTModern * 2 Then 'Linear interpolation from present value to future model
                dblCorrection = 102 + (1618 + NDeltaTModern * 2 - 2000) / 100 * (102 + (1618 + NDeltaTModern * 2 - 2000) / 100 * 25.3) - dblDeltaT(NDeltaTModern - 1)
                DeltaT = DeltaT + dblCorrection / 100 * (dblJYear - 1718 - NDeltaTModern * 2) 'To maintain continuity
            End If
        ElseIf dblJYear < 1620 Then 'No data between 1600 and 1620 so linear interpolation
            dblY(0) = dblPastDeltaT(42)
            dblY(1) = dblDeltaT(0)
            DeltaT = Interp(dblY, (dblJYear - 1600) / 20, 2)
        ElseIf dblJYear < 1621 Then 'Linear interpolation between first two points of second table
            dblY(0) = dblDeltaT(0)
            dblY(1) = dblDeltaT(1)
            DeltaT = Interp(dblY, (dblJYear - 1620) / 2, 2)
        ElseIf dblJYear > 1617 + NDeltaTModern * 2 Then 'Linear interpolation between last two points of last table
            dblY(0) = dblDeltaT(190)
            dblY(1) = dblDeltaT(191)
            DeltaT = Interp(dblY, (dblJYear - 1616 - NDeltaTModern * 2) / 2, 2)
        Else 'Three point interpolation of second table
            dblY(0) = dblDeltaT((dblJYear - 1621) \ 2)
            dblY(1) = dblDeltaT((dblJYear - 1621) \ 2 + 1)
            dblY(2) = dblDeltaT((dblJYear - 1621) \ 2 + 2)
            DeltaT = Interp(dblY, dblJYear / 2 - Int((dblJYear - 1) / 2), 3)
        End If
        blnFinished = blnFromUTC Or Abs(DeltaT - dblOldDeltaT) < 100 'if date in ephemeris time and year very large an iteration is required to maintain consistency
        dblOldDeltaT = DeltaT
    Wend
End Function
