VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGregorian"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use
'Most of the algorithms in this module are from
'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.

Const GregorianEpoch = 1

Function GregorianLeapYear(intYear As Integer) As Boolean
'Is it a leap year
    GregorianLeapYear = (MyMod(intYear, 4) = 0 And (MyMod(intYear, 100) > 0 Or MyMod(intYear, 400) = 0))
End Function
Friend Function gregorian_fixed(udtDate As MyDate) As Long
'Returns fixed radix date
    Static lastDate As MyDate
    Static lastResult As Double
    
    If lastDate.intDateOfMonth = udtDate.intDateOfMonth And lastDate.intMonth = udtDate.intMonth And lastDate.intYear = udtDate.intYear Then
        gregorian_fixed = lastResult
    Else
        gregorian_fixed = GregorianEpoch - 1 + 365 * (CLng(udtDate.intYear) - 1) + Int((udtDate.intYear - 1) / 4) - Int((udtDate.intYear - 1) / 100) + Int((udtDate.intYear - 1) / 400) + Int((367 * CLng(udtDate.intMonth) - 362) / 12) + udtDate.intDateOfMonth
        If udtDate.intMonth > 2 And GregorianLeapYear(udtDate.intYear) Then
            gregorian_fixed = gregorian_fixed - 1
        ElseIf udtDate.intMonth > 2 Then
            gregorian_fixed = gregorian_fixed - 2
        End If
        lastDate.intDateOfMonth = udtDate.intDateOfMonth
        lastDate.intMonth = udtDate.intMonth
        lastDate.intYear = udtDate.intYear
        lastResult = gregorian_fixed
    End If
End Function
Function fixed_gregorianYear(lngDate As Long) As Long
'Returns the Gregorian year
    Dim lngD0 As Long
    Dim lngD1 As Long
    Dim lngD2 As Long
    Dim lngD3 As Long
    Dim intN400 As Integer
    Dim intN100 As Integer
    Dim intN4 As Integer
    Dim intN1 As Integer
    Static lastDate As Double
    Static lastResult As Long
    
    If lngDate <> 0 And lngDate = lastDate Then
        fixed_gregorianYear = lastResult
    Else
        lngD0 = lngDate - GregorianEpoch
        intN400 = Int(lngD0 / 146097)
        lngD1 = MyMod(lngD0, 146097)
        intN100 = Int(lngD1 / 36524)
        lngD2 = MyMod(lngD1, 36524)
        intN4 = Int(lngD2 / 1461)
        lngD3 = MyMod(lngD2, 1461)
        intN1 = Int(lngD3 / 365)
        fixed_gregorianYear = 400# * intN400 + 100 * intN100 + 4 * intN4 + intN1
        If intN1 < 4 And intN100 < 4 Then fixed_gregorianYear = fixed_gregorianYear + 1
        lastDate = lngDate
        lastResult = fixed_gregorianYear
    End If
End Function
Friend Sub fixed_gregorian(lngDate As Long, udtDateOut As MyDate, blnValid As Boolean)
'Convert a date number to a gregorian date
    Dim lngPriorDays As Long
    Dim intCorrection As Integer
    Dim lngYear As Long
    Static lastDateOut As MyDate
    Static lastDate As Double
    
    udtDateOut.enmDateType = gregorian
    If lastDate = lngDate And lngDate <> 0 Then
        udtDateOut.intDateOfMonth = lastDateOut.intDateOfMonth
        udtDateOut.intMonth = lastDateOut.intMonth
        udtDateOut.intYear = lastDateOut.intYear
    Else
        lngYear = fixed_gregorianYear(lngDate)
        If Abs(lngYear) > 32765 Then
            blnValid = False
        Else
            udtDateOut.intYear = lngYear
            udtDateOut.intMonth = 1
            udtDateOut.intDateOfMonth = 1
            lngPriorDays = lngDate - gregorian_fixed(udtDateOut)
            udtDateOut.intMonth = 3
            If lngDate >= gregorian_fixed(udtDateOut) Then
                If GregorianLeapYear(udtDateOut.intYear) Then
                    intCorrection = 1
                Else
                    intCorrection = 2
                End If
            Else
                intCorrection = 0
            End If
            udtDateOut.intMonth = Int((12 * (lngPriorDays + intCorrection) + 373) / 367)
            udtDateOut.intDateOfMonth = Int(lngDate - gregorian_fixed(udtDateOut) + 1)
            lastDate = lngDate
            lastDateOut.intDateOfMonth = udtDateOut.intDateOfMonth
            lastDateOut.intMonth = udtDateOut.intMonth
            lastDateOut.intYear = udtDateOut.intYear
        End If
    End If
End Sub
Friend Function MonthName(intMonth As Integer) As String
'Returns the name of the month (Gregorian or Julian)
    Static lastMonth As Integer
    Static lastMonthName As String
    
    If intMonth = lastMonth Then
        MonthName = lastMonthName
    Else
        Select Case intMonth
        Case 1
            MonthName = "January"
        Case 2
            MonthName = "February"
        Case 3
            MonthName = "March"
        Case 4
            MonthName = "April"
        Case 5
            MonthName = "May"
        Case 6
            MonthName = "June"
        Case 7
            MonthName = "July"
        Case 8
            MonthName = "August"
        Case 9
            MonthName = "September"
        Case 10
            MonthName = "October"
        Case 11
            MonthName = "November"
        Case 12
            MonthName = "December"
        Case Else
            MonthName = CStr(intMonth)
        End Select
        lastMonth = intMonth
        lastMonthName = MonthName
    End If
End Function
Private Function DaysInGregorianMonth(intMonth As Integer, intYear As Integer) As Integer
'Last date of the month
    If intMonth = 4 Or intMonth = 6 Or intMonth = 9 Or intMonth = 11 Then
        DaysInGregorianMonth = 30
    ElseIf intMonth = 2 Then
        If GregorianLeapYear(intYear) Then
            DaysInGregorianMonth = 29
        Else
            DaysInGregorianMonth = 28
        End If
    Else
        DaysInGregorianMonth = 31
    End If
End Function
Friend Function BeforeOnAfter(intDayOfWeek As Integer, intDateOfMonth As Integer, intWeekNo As Integer, intMonth As Integer, dblDate As Double) As Integer
'Returns -1 before, 0 on, 1 after
'intWeekNo is the number of weeks after the date on the intDayOfWeek defined
'intWeekNo 0 means on that date and intWeekNo 5 means the last week of the month
    Dim udtDateIn As MyDate
    Dim intDateNo As Integer
    Dim intdateno2 As Integer
    Dim blnValid As Boolean
    
    blnValid = True
    Call fixed_gregorian(Int(dblDate), udtDateIn, blnValid)
    'Months
    If intMonth > udtDateIn.intMonth Then
        BeforeOnAfter = -1
    ElseIf intMonth < udtDateIn.intMonth Then
        BeforeOnAfter = 1
    Else 'If same month
        If intWeekNo = 0 Then
            intDateNo = intDateOfMonth
        ElseIf intWeekNo = 5 Then 'Last week of month
            intDateNo = DaysInGregorianMonth(udtDateIn.intMonth, udtDateIn.intYear) - 6
        Else 'Count the weeks from the date, 1 is 1st week
            intDateNo = intDateOfMonth + 7 * (intWeekNo - 1)
        End If
        If intWeekNo > 0 Then 'Select the day
            intdateno2 = udtDateIn.intDateOfMonth
            udtDateIn.intDateOfMonth = intDateNo
            intDateNo = intDateNo + MyMod(intDayOfWeek - fixed_DayOfWeek(frmDate.EvaluateDate(udtDateIn)), 7)
            udtDateIn.intDateOfMonth = intdateno2
        End If
        If udtDateIn.intDateOfMonth > intDateNo Then
            BeforeOnAfter = 1
        ElseIf udtDateIn.intDateOfMonth = intDateNo Then
            BeforeOnAfter = 0
        Else 'Before the date
            BeforeOnAfter = -1
        End If
    End If
End Function
Function fixed_DayOfWeek(dblDate As Double) As Integer
    'Modulus of fixed radix day number
    '0 = Sunday
    fixed_DayOfWeek = MyMod(Int(dblDate), 7)
End Function

