VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsHebrew"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use
'Most of the algorithms in this module are from
'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.

Const HebrewEpoch = -1373427
Function HebrewLeapYear(intYear As Integer) As Boolean
'Is it a leap year
    'The leap year occurs 7 times during the 19 year cycle
    'On the 3rd, 6th, 8th, 11th, 14th, 17th and 19th year
    HebrewLeapYear = MyMod(7 * CLng(intYear) + 1, 19) < 7
End Function
Function MonthsInHebrewYear(intYear As Integer) As Integer
'Returns the number of months in the year
    If HebrewLeapYear(intYear) Then
        MonthsInHebrewYear = 13
    Else
        MonthsInHebrewYear = 12
    End If
End Function
Function HebrewElapsedDays(intYear As Integer) As Long
'Number of days since the epoch at the start of the year
'This does not take account of Gatrad or Btu-takpat
    Dim lngMonthsElapsed As Long
    Dim lngPartsElapsed As Long
    Dim lngHoursElapsed As Long
    Dim lngDaysElapsed As Long
    Static lastYear As Integer
    Static lastResult As Long
    
    If intYear = lastYear And intYear <> 0 Then
        HebrewElapsedDays = lastResult
    Else
        lngMonthsElapsed = Int((235 * CLng(intYear) - 234) / 19)
        lngPartsElapsed = 204 + 793 * MyMod(lngMonthsElapsed, 1080)
        lngHoursElapsed = 11 + 12 * lngMonthsElapsed + 793 * Int(lngMonthsElapsed / 1080) + Int(lngPartsElapsed / 1080)
        lngDaysElapsed = 29 * lngMonthsElapsed + Int(lngHoursElapsed / 24)
        If MyMod(3 * (lngDaysElapsed + 1), 7) < 3 Then
            HebrewElapsedDays = lngDaysElapsed + 1 'Lo Adu Rosh
        Else
            HebrewElapsedDays = lngDaysElapsed
        End If
        lastYear = intYear
        lastResult = HebrewElapsedDays
    End If
End Function
Function HebrewNewYearDelay(intYear As Integer) As Integer
'Calculates the dechiyot due to Gatrad and Btu-takpat
    Dim lngNY0 As Long
    Dim lngNY1 As Long
    Dim lngNY2 As Long
    Static lastYear As Integer
    Static lastResult As Integer
    
    If intYear = lastYear And intYear <> 0 Then
        HebrewNewYearDelay = lastResult
    Else
        lngNY0 = HebrewElapsedDays(intYear - 1)
        lngNY1 = HebrewElapsedDays(intYear)
        lngNY2 = HebrewElapsedDays(intYear + 1)
        If lngNY2 - lngNY1 = 356 Then
            HebrewNewYearDelay = 2
        ElseIf lngNY1 - lngNY0 = 382 Then
            HebrewNewYearDelay = 1
        Else
            HebrewNewYearDelay = 0
        End If
        lastYear = intYear
        lastResult = HebrewNewYearDelay
    End If
End Function
Function HebrewNewYear(intYear As Integer) As Long
'Returns the number of days elapsed till the New Year taking into account all the dechiyot
    HebrewNewYear = HebrewEpoch + HebrewElapsedDays(intYear) + HebrewNewYearDelay(intYear)
End Function
Function DaysInHebrewYear(intYear As Integer) As Integer
'Returns the number of days in a Hebrew year
    Static lastYear As Integer
    Static lastResult As Integer
    
    If intYear = lastYear And intYear <> 0 Then
        DaysInHebrewYear = lastResult
    Else
        DaysInHebrewYear = HebrewNewYear(intYear + 1) - HebrewNewYear(intYear)
        lastYear = intYear
        lastResult = DaysInHebrewYear
    End If
End Function
Function ShortKislev(intYear As Integer) As Boolean
'Does Kislev have 29 days?
    Static lastYear As Integer
    Static lastResult As Integer
    
    If intYear = lastYear And intYear <> 0 Then
        ShortKislev = lastResult
    Else
        ShortKislev = (DaysInHebrewYear(intYear) Mod 10 = 3)
        lastYear = intYear
        lastResult = ShortKislev
    End If
End Function
Function LongCheshvan(intYear As Integer) As Boolean
'Does Cheshvan have 30 days?
    Static lastYear As Integer
    Static lastResult As Integer
    
    If intYear = lastYear And intYear <> 0 Then
        LongCheshvan = lastResult
    Else
        LongCheshvan = (DaysInHebrewYear(intYear) Mod 10 = 5)
        lastYear = intYear
        lastResult = LongCheshvan
    End If
End Function
Function LastDayOfHebrewMonth(intMonth As Integer, intYear As Integer) As Integer
'Returns the last date of the month, 29 or 30
    Static lastMonth As Integer
    Static lastYear As Integer
    Static lastResult As Integer
    
    If intMonth = lastMonth And intYear = lastYear Then
        LastDayOfHebrewMonth = lastResult
    Else
        If intMonth = 2 Or intMonth = 4 Or intMonth = 6 Or intMonth = 10 Or intMonth = 13 Or intMonth = 12 And Not HebrewLeapYear(intYear) Or intMonth = 8 And Not LongCheshvan(intYear) Or intMonth = 9 And ShortKislev(intYear) Then
            'Iyar, Tammuz, Ellul, Shevat, Adar 2 and sometimes Cheshvan, Kislev and Adar
            LastDayOfHebrewMonth = 29
        Else
            LastDayOfHebrewMonth = 30
        End If
        lastMonth = intMonth
        lastYear = intYear
        lastResult = LastDayOfHebrewMonth
    End If
End Function
Friend Function hebrew_fixed(udtDate As MyDate) As Long
'Convert a Hebrew date to fixed radix
    Dim intM As Integer
    Static lastDate As MyDate
    Static lastResult As Double
    Dim udtDate2 As MyDate
    Dim cSun As New clsSun
    Dim dblSS As Double
    
    If udtDate.intDateOfMonth = lastDate.intDateOfMonth And udtDate.intMonth = lastDate.intMonth And udtDate.intYear = lastDate.intYear Then
        hebrew_fixed = lastResult
    Else
        hebrew_fixed = HebrewNewYear(udtDate.intYear) + udtDate.intDateOfMonth - 1
        If udtDate.intMonth < 7 Then
            For intM = 7 To MonthsInHebrewYear(udtDate.intYear)
                hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, udtDate.intYear)
            Next
            For intM = 1 To udtDate.intMonth - 1
                hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, udtDate.intYear)
            Next
        Else
            For intM = 7 To udtDate.intMonth - 1
                hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, udtDate.intYear)
            Next
        End If
        lastDate.intDateOfMonth = udtDate.intDateOfMonth
        lastDate.intMonth = udtDate.intMonth
        lastDate.intYear = udtDate.intYear
        lastResult = hebrew_fixed
    End If
End Function
Friend Sub fixed_hebrew(lngDate As Long, udtDate As MyDate, blnValid As Boolean)
'Convert a fised radix date to the Hebrew Calendar
    Dim lngApprox As Double
    Dim intStart As Integer
    Static lastDate As Long
    Static lastudtDate As MyDate
    Dim udtDate2 As MyDate
    Dim cSun As New clsSun
    Dim dblSS As Double
    Dim udtTimeZero As Mytime
    
    udtDate.enmDateType = Hebrew
    udtTimeZero.enmTimeType = meansolar
    udtTimeZero.udtDms.intDegreesHours = 6
    udtTimeZero.udtDms.intMinutes = 0
    udtTimeZero.udtDms.dblSeconds = 0
    If lngDate = lastDate And lastDate <> 0 Then
        udtDate.intDateOfMonth = lastudtDate.intDateOfMonth
        udtDate.intMonth = lastudtDate.intMonth
        udtDate.intYear = lastudtDate.intYear
    Else
        lngApprox = Int((lngDate - HebrewEpoch) * 98496# / 35975351) + 1
        If Abs(lngApprox > 32765) Then
            blnValid = False
        Else
            udtDate.intMonth = 7
            udtDate.intDateOfMonth = 1
            udtDate.intYear = lngApprox - 1
            While hebrew_fixed(udtDate) <= lngDate
                udtDate.intYear = udtDate.intYear + 1
            Wend
            udtDate.intYear = udtDate.intYear - 1
            udtDate.intMonth = 1
            If dblDate < hebrew_fixed(udtDate) Then
                udtDate.intMonth = 7
            Else
                udtDate.intMonth = 1
            End If
            udtDate.intDateOfMonth = LastDayOfHebrewMonth(udtDate.intMonth, udtDate.intYear)
            While lngDate > hebrew_fixed(udtDate)
                udtDate.intMonth = udtDate.intMonth + 1
                udtDate.intDateOfMonth = LastDayOfHebrewMonth(udtDate.intMonth, udtDate.intYear)
            Wend
            udtDate.intDateOfMonth = 1
            udtDate.intDateOfMonth = lngDate - hebrew_fixed(udtDate) + 1
            lastDate = lngDate
            lastudtDate.intDateOfMonth = udtDate.intDateOfMonth
            lastudtDate.intMonth = udtDate.intMonth
            lastudtDate.intYear = udtDate.intYear
        End If
    End If
End Sub
Friend Function MonthName(intMonth As Integer, Optional intYear As Integer = -32767) As String
'Returns the name of the Hebrew Month
    Static lastMonth As Integer
    Static lastYear As Integer
    Static lastResult As String
    
    If intMonth = lastMonth And intYear = lastYear Then
        MonthName = lastResult
    Else
        Select Case intMonth
        Case 1
            MonthName = "Nisan"
        Case 2
            MonthName = "Iyar"
        Case 3
            MonthName = "Sivan"
        Case 4
            MonthName = "Tammuz"
        Case 5
            MonthName = "Av"
        Case 6
            MonthName = "Ellul"
        Case 7
            MonthName = "Tishri"
        Case 8
            MonthName = "Cheshvan"
        Case 9
            MonthName = "Kislev"
        Case 10
            MonthName = "Tevet"
        Case 11
            MonthName = "Shevat"
        Case 12
            If intYear = -32767 Then 'Year not defined
                MonthName = "Adar (1)"
            ElseIf HebrewLeapYear(intYear) Then
                MonthName = "Adar 1"
            Else
                MonthName = "Adar"
            End If
        Case 13
            MonthName = "Adar 2"
        Case Else
            MonthName = CStr(intMonth)
        End Select
        lastMonth = intMonth
        lastYear = intYear
        lastResult = MonthName
    End If
End Function
Friend Sub StartEndDay(dblFix As Double, dblStart As Double, dblEnd As Double, intDayShift As Integer)
'Get start and end of Hebrew or Islamic day for a particular time
'and provide an adjustment for the number of days from the UTC day
    Dim blnNeverRises As Boolean
    Dim blnNeverSets As Boolean
    Dim cHBody As New clsHBody
    Dim cLocal As clsLocalTime
    
    Call cHBody.RiseSet("Sun", Int(dblFix) - 1, True, dblStart, True, blnNeverSets, blnNeverRises)
    Call cHBody.RiseSet("Sun", Int(dblFix), True, dblEnd, True, blnNeverSets, blnNeverRises)
    intDayShift = 0
    While dblStart > dblFix 'While the start is after the moment in time (dblFix)
        intDayShift = intDayShift - 1 'Go back a day
        dblEnd = dblStart
        Call cHBody.RiseSet("Sun", Int(dblFix) - 1 + intDayShift, True, dblStart, True, blnNeverSets, blnNeverRises)
    Wend
    While dblEnd < dblFix 'While the end is before the moment in time (dblFix)
        intDayShift = intDayShift + 1 'Go forward a day
        dblStart = dblEnd
        Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift, True, dblEnd, True, blnNeverSets, blnNeverRises)
    Wend
End Sub
Friend Sub AdjustTime(lngDate As Long, dblTime As Double, blnValid As Boolean, blnSecondTime As Boolean)
'Adjust the supplied time for a Hebrew or Islamic date
    Dim intDayShift As Integer
    Dim dblSet As Double
    Dim blnNeverRises As Boolean
    Dim blnNeverSets As Boolean
    Dim cHBody As New clsHBody
    
    intDayShift = 0
    'If before sunset the local previous day then add a day
    Call cHBody.RiseSet("Sun", lngDate - 1, True, dblSet, True, blnNeverRises, blnNeverSets)
    While dblTime < dblSet - lngDate
        dblTime = dblTime + 1
        intDayShift = intDayShift + 1
        Call cHBody.RiseSet("Sun", lngDate - 1 - intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets)
    Wend
    'If after sunset today, subtract a day
    Call cHBody.RiseSet("Sun", lngDate - 1, True, dblSet, True, blnNeverRises, blnNeverSets)
    While dblTime > dblSet - lngDate + 1 ' And Not blnSecondTime
        dblTime = dblTime - 1
        If blnSecondTime Then intDayShift = intDayShift - 1
        Call cHBody.RiseSet("Sun", lngDate - 1 - intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets)
        'If blnSecondTime Then intDayShift = intDayShift - 1
    Wend
    Call cHBody.RiseSet("Sun", lngDate + intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets)
    If dblTime > dblSet - lngDate - intDayShift Then
        blnValid = False
    End If
End Sub

