VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsPlanet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use
'Some of the algorithms from
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87

Const AU = 149597870 'Astronomical unit in km
Const TwoPi = 6.28318530717959

Friend Sub HelioPlanet(strPlanet As String, dblFix As Double, dblL, dblB, dblR)
    'Calculate heliocentric coordinates of a planet
    'Added save routine to save repeat calculations
    Dim cHBody As New clsHBody
    
    Select Case strPlanet
        Case "Mercury":
            If blnMercury And Not blnChangedInput And dblFix = udtMercurySave.Fix Then
                With udtMercurySave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnMercury Then
                    Call cHBody.ReadHBody(strPlanet, udtMercury, lngMoonLR, lngMoonB, udtStar)
                    blnMercury = True
                End If
                Call helioplanet2(udtMercury, dblFix, dblL, dblB, dblR)
                With udtMercurySave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case "Venus":
            If blnVenus And Not blnChangedInput And dblFix = udtVenusSave.Fix Then
                With udtVenusSave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnVenus Then
                    Call cHBody.ReadHBody(strPlanet, udtVenus, lngMoonLR, lngMoonB, udtStar)
                    blnVenus = True
                End If
                Call helioplanet2(udtVenus, dblFix, dblL, dblB, dblR)
                With udtVenusSave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case "Earth":
            If blnEarth And Not blnChangedInput And dblFix = udtEarthSave.Fix Then
                With udtEarthSave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnEarth Then
                    Call cHBody.ReadHBody(strPlanet, udtEarth, lngMoonLR, lngMoonB, udtStar)
                    blnEarth = True
                End If
                Call helioplanet2(udtEarth, dblFix, dblL, dblB, dblR)
                With udtEarthSave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case "Mars":
            If blnMars And Not blnChangedInput And dblFix = udtMarsSave.Fix Then
                With udtMarsSave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnMars Then
                    Call cHBody.ReadHBody(strPlanet, udtMars, lngMoonLR, lngMoonB, udtStar)
                    blnMars = True
                End If
                Call helioplanet2(udtMars, dblFix, dblL, dblB, dblR)
                With udtMarsSave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case "Jupiter":
            If blnJupiter And Not blnChangedInput And dblFix = udtJupiterSave.Fix Then
                With udtJupiterSave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnJupiter Then
                    Call cHBody.ReadHBody(strPlanet, udtJupiter, lngMoonLR, lngMoonB, udtStar)
                    blnJupiter = True
                End If
                Call helioplanet2(udtJupiter, dblFix, dblL, dblB, dblR)
                With udtJupiterSave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case "Saturn":
            If blnSaturn And Not blnChangedInput And dblFix = udtSaturnSave.Fix Then
                With udtSaturnSave
                    dblL = .L
                    dblB = .B
                    dblR = .R
                End With
            Else
                If Not blnSaturn Then
                    Call cHBody.ReadHBody(strPlanet, udtSaturn, lngMoonLR, lngMoonB, udtStar)
                    blnSaturn = True
                End If
                Call helioplanet2(udtSaturn, dblFix, dblL, dblB, dblR)
                With udtSaturnSave
                    .Fix = dblFix
                    .L = dblL
                    .B = dblB
                    .R = dblR
                End With
            End If
        Case Else
            MsgBox ("Invalid planet name in helioplanet")
    End Select
End Sub
Friend Sub helioplanet2(udtPars As VSOP, dblFix As Double, dblL, dblB, dblR)
    Dim dblTau As Double
    Dim intI As Long
    Dim intJ As Long
    Dim dblAngle As Double
    Dim cJDE As New clsJDE
    
    dblTau = (cJDE.fixed_jde(dblFix) - 2451545) / 365250
    dblL = 0
    For intI = udtPars.NL To 0 Step -1
        dblL = dblL * dblTau
        For intJ = 1 To udtPars.L(intI).N
            'Reduce the angle to one circle (0 to 2 pi)
            'Written inline to increase speed
            dblAngle = udtPars.L(intI).B(intJ) + dblTau * udtPars.L(intI).C(intJ)
            dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
            dblL = dblL + udtPars.L(intI).A(intJ) * Cos(dblAngle)
        Next
    Next
    dblL = FracCirc(dblL / 100000000# * RADCON)
    dblB = 0
    For intI = udtPars.NB To 0 Step -1
        dblB = dblB * dblTau
        For intJ = 1 To udtPars.B(intI).N
            dblAngle = udtPars.B(intI).B(intJ) + dblTau * udtPars.B(intI).C(intJ)
            dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
            dblB = dblB + udtPars.B(intI).A(intJ) * Cos(dblAngle)
        Next
    Next
    dblB = FracCirc(dblB / 100000000# * RADCON)
    If dblB >= 180 Then dblB = dblB - 360
    dblR = 0
    For intI = udtPars.NR To 0 Step -1
        dblR = dblR * dblTau
        For intJ = 1 To udtPars.R(intI).N
            dblAngle = udtPars.R(intI).B(intJ) + dblTau * udtPars.R(intI).C(intJ)
            dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
            dblR = dblR + udtPars.R(intI).A(intJ) * Cos(dblAngle)
        Next
    Next
    dblR = dblR / 100000000# * 149597870
End Sub

Friend Sub AnalyzeSightings()
    Const NColumns = 24 'Number of coulumns in sighting table
    Const strSeen = "Y" 'Analyse positive (Y) or negative (N) sightings
    Const MakeTable = False 'Just make a table for publication
    Const Verbose = True 'Include observer's name and date in output
    Const photometric = True
    Dim intI As Integer
    Dim intRow As Integer
    Dim intNRows As Integer
    Dim intBase As Integer
    Dim lngFix As Long
    Dim strEntry() As String
    Dim blnValid As Boolean
    Dim dblP As Double 'Probability of visibility
    Dim blnNew As Boolean
    Dim strFName As String
    Dim intIndex As Integer
    Dim blnAccept As Boolean
    Dim strPlanet As String
    Dim udtSun As HBody
    Dim udtPlanet As HBody
    Dim cSun As New clsSun
    Dim cHBody As New clsHBody
    
    If Not blnStarRead Then 'First time, read table
        Call cHBody.ReadHBody("Star", udtMercury, lngMoonLR, lngMoonB, udtStar)
        blnStarRead = True
    End If
    strFName = "PlanetSightings.txt"
    intIndex = 16
    Open strFName For Input Access Read As 2
    intRow = 0
    While Not EOF(2)
        ReDim Preserve strEntry(NColumns * (intRow + 1) - 1)
        For intI = 0 To NColumns - 1
             If Not EOF(2) Then Input #2, strEntry(intRow * NColumns + intI)
        Next
        intRow = intRow + 1
    Wend
    Close 2
    intNRows = intRow
    Open "Output.txt" For Output Access Write As 2
    For intRow = 0 To intNRows - 1
        intBase = intRow * NColumns
        strPlanet = strEntry(intBase + 9)
        If MakeTable Then
            blnAccept = (strEntry(intBase + 12) = "Y" And strEntry(intBase + 13) = "N" And strEntry(intBase + 14) = "N" And strEntry(intBase + 15) = "N" Or strEntry(intBase + 17) = "Y" And strEntry(intBase + 18) = "N" And strEntry(intBase + 19) = "N" And strEntry(intBase + 20) = "N") And (strPlanet = "Mercury" Or strPlanet = "Venus" Or strPlanet = "Mars" Or strPlanet = "Jupiter" Or strPlanet = "Saturn" Or strPlanet = "Sirius")
            If blnAccept Then
                Call frmLocation.GetPositionFromName(strEntry(intBase + 3), strEntry(intBase + 4), strEntry(intBase + 5), blnValid)
                If strEntry(intBase + 12) = "Y" And strEntry(intBase + 13) = "N" And strEntry(intBase + 14) = "N" And strEntry(intBase + 15) = "N" And strEntry(intBase + 17) = "Y" And strEntry(intBase + 18) = "N" And strEntry(intBase + 19) = "N" And strEntry(intBase + 20) = "N" Then
                    Print #2, strEntry(intBase) & "," & udtLocation.dblLatitude & "," & udtLocation.dblLongitude & "," & udtLocation.dblHeight & "," & strEntry(intBase + 7) & "," & strEntry(intBase + 9) & "," & strEntry(intBase + 10) & "," & strEntry(intBase + 8) & "," & strEntry(intBase + 11) & "," & strEntry(intBase + 16) & "," & strEntry(intBase + 21) & "," & strEntry(intBase + 22) & "," & strEntry(intBase + 23)
                ElseIf strEntry(intBase + 12) = "Y" And strEntry(intBase + 13) = "N" And strEntry(intBase + 14) = "N" And strEntry(intBase + 15) = "N" Then
                    Print #2, strEntry(intBase) & "," & udtLocation.dblLatitude & "," & udtLocation.dblLongitude & "," & udtLocation.dblHeight & "," & strEntry(intBase + 7) & "," & strEntry(intBase + 9) & "," & strEntry(intBase + 10) & "," & strEntry(intBase + 8) & "," & strEntry(intBase + 11) & ",," & strEntry(intBase + 21) & "," & strEntry(intBase + 22) & "," & strEntry(intBase + 23)
                Else
                    Print #2, strEntry(intBase) & "," & udtLocation.dblLatitude & "," & udtLocation.dblLongitude & "," & udtLocation.dblHeight & "," & strEntry(intBase + 7) & "," & strEntry(intBase + 9) & "," & strEntry(intBase + 10) & "," & strEntry(intBase + 8) & ",," & strEntry(intBase + 16) & "," & strEntry(intBase + 21) & "," & strEntry(intBase + 22) & "," & strEntry(intBase + 23)
                End If
            End If
        Else
            Call frmTime.TimeFromString(strEntry(intBase + 11), strEntry(intBase + 8), blnValid)
            If Not blnValid Then Call frmTime.TimeFromString(strEntry(intBase + 16), strEntry(intBase + 8), blnValid)
            Call frmDate.DateFromString(strEntry(intBase + 7), blnValid)
            dblDate = frmDate.EvaluateDate(udtDate)
            Call frmTime.EvaluateTime(blnValid)
            lngFix = frmDate.ToLngDate(dblDate + dblTime)
            blnAccept = (strEntry(intBase + 10) = strSeen) And (strEntry(intBase + 12) = "Y" And strEntry(intBase + 13) = "N" And strEntry(intBase + 14) = "N" And strEntry(intBase + 15) = "N" And cHBody.EveningSight(strPlanet, lngFix) Or strEntry(intBase + 17) = "Y" And strEntry(intBase + 18) = "N" And strEntry(intBase + 19) = "N" And strEntry(intBase + 20) = "N" And Not cHBody.EveningSight(strPlanet, lngFix)) And (strPlanet = "Mercury" Or strPlanet = "Venus" Or strPlanet = "Mars" Or strPlanet = "Jupiter" Or strPlanet = "Saturn" Or strPlanet = "Sirius")
            If blnAccept And blnValid Then
            'Accept those cases where it was seen by an experienced observer without errors or interference from clouds
                Call frmLocation.GetPositionFromName(strEntry(intBase + 3), strEntry(intBase + 4), strEntry(intBase + 5), blnValid)
                If blnValid Then
                    dblP = cHBody.HBProbVis(strPlanet, dblDate + dblTime)
                    If Verbose Then
                        Print #2, strEntry(intBase), strEntry(intBase + 7), dblP
                    Else
                        Print #2, dblP
                    End If
                End If
            End If
        End If
    Next
    Close 2
End Sub
Sub DrawStar(sngSize As Single, curDisplay As Object)
    Dim sngX As Single
    Dim sngY As Single
    Dim intI As Integer
    
    sngX = curDisplay.CurrentX
    sngY = curDisplay.CurrentY
    curDisplay.FillColor = vbBlack
    curDisplay.FillStyle = vbFSSolid
    curDisplay.Circle Step(0, 0), sngSize / 2
    curDisplay.FillColor = vbWhite
End Sub
