VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSky"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Const TwoPi = 6.28318530717959
Friend Function DeltaM(dblFix As Double, dblAlt As Double, dblAz As Double, intMode As Integer) As Double
'Calculate the extinction of stellar magnitude
    Dim dblKR As Double 'Rayleigh extinction coefficient
    Dim dblKOz As Double 'Ozone extinction coefficient
    Dim dblKA As Double 'Aerosol extinction coefficient
    Dim dblKW As Double 'Water extinction coefficient
    Dim dblXG As Double 'Path length for gas (Reyleigh and water)
    Dim dblXOz As Double 'Path length for ozone
    Dim dblXA As Double 'Path length for aerosol
    Dim dblZ As Double 'azimuth angle (90 - dblAlt)
    Dim dblZR As Double 'azimuth angle corrected for refraction
    Dim udtSun As HBody 'Sun's parameters
    Dim blnRefValid As Boolean 'Refraction valid if not too far below the horizon
    Dim cHBody As New clsHBody
    
    Call cHBody.TrueHBody("Sun", dblFix, udtSun)
    dblZ = 90 - dblAlt
    dblZR = 90 - dblAlt - cHBody.Refract(dblAlt, True, udtWeatherNow, blnRefValid)
    If Not blnRefValid Then dblZR = dblZ
    Call GetK(dblFix, intMode, udtSun.RATopo, udtSun.HourAngTopo, dblKA, dblKOz, dblKR, dblKW)
    Call GetX(dblFix, intMode, dblZ, dblZR, dblXA, dblXOz, dblXG)
    DeltaM = (dblKR + dblKW) * dblXG + dblKOz * dblXOz + dblKA * dblXA
End Function
Friend Sub GetX(dblFix As Double, intMode As Integer, dblZ As Double, dblZR As Double, dblXA As Double, dblXOz As Double, dblXG As Double)
'Get path lengths in atmospheres
    Select Case intMode
    Case 0 'Shcaefer 1993, as suggested by Hoffman & Kaatz, Yodei Bina, in press
        dblXG = 1 / (cosD(dblZ) + 0.0286 * Exp(-10.48 * cosD(dblZ)))
        dblXOz = 1 / Sqr(1 - 0.993758 * Square(sinD(dblZ)))
        dblXA = 1 / (cosD(dblZ) + 0.0122 * Exp(-24.49 * cosD(dblZ)))
    Case 1 'according to B. E. Schaefer, Sky Telesc., May 1998, 57.
        dblXG = 1 / (cosD(dblZ) + 0.286 * Exp(-10.5 * cosD(dblZ)))
        dblXOz = 1 / Sqr(1 - Square(sinD(dblZ) / (1 + 20 / 6378)))
        dblXA = 1 / (cosD(dblZ) + 0.0123 * Exp(-24.5 * cosD(dblZ)))
    Case 2 'Shcaefer 1993, as suggested by Hoffman & Kaatz, Yodei Bina, in press
        dblXG = 1 / (cosD((dblZ + dblZR) / 2) + 0.0286 * Exp(-10.48 * cosD((dblZ + dblZR) / 2)))
        dblXOz = 1 / Sqr(1 - 0.993758 * Square(sinD(dblZ)))
        dblXA = 1 / (cosD(dblZR) + 0.0122 * Exp(-24.49 * cosD(dblZR)))
    End Select
End Sub
Friend Sub GetK(dblFix As Double, intMode As Integer, dblRA As Double, dblH As Double, dblKA As Double, dblKOz As Double, dblKR As Double, dblKW As Double)
'Get extinction coefficients
    Select Case intMode
    Case 0, 2 'Schaefer 1993, as suggested by Hoffman & Kaatz, Yodei Binah, 1, 2001
        dblKR = 0.1066 * Exp(-udtLocation.dblHeight / 8200)
        dblKOz = 0.010333 * (3 + 0.4 * (udtLocation.dblLatitude / RADCON * cosD(dblRA) - cosD(3 * udtLocation.dblLatitude)))
        dblKA = 0.12 * Exp(-udtLocation.dblHeight / 1500) * (1 - 0.32 / Log(udtWeatherNow.dblHumidity / 100)) ^ 1.3333 * (1 + 0.33 * sinD(dblRA) * Sgn(udtLocation.dblLatitude))
        If intMode = 2 Then 'correct for daily variations
            dblKA = dblKA * (1 + 0.43 * cosD(dblH - 38) * cosD(udtLocation.dblLatitude))
        End If
        dblKW = 0 'Water vapor combined with aerosols by this method
    Case 1 'Schaefer 1998
        dblKR = 0.1066 * Exp(-udtLocation.dblHeight / 8200)
        dblKOz = 0.010333 * (3 + 0.4 * (udtLocation.dblLatitude / RADCON * cosD(dblRA) - cosD(3 * udtLocation.dblLatitude)))
        'Added the factor 0.33 which is obviously an error in the published paper                                          here
        dblKA = 0.1 * Exp(-udtLocation.dblHeight / 1500) * (1 - 0.32 / Log(udtWeatherNow.dblHumidity / 100)) ^ 1.33 * (1 + 0.33 * sinD(dblRA) * Sgn(udtLocation.dblLatitude))
        dblKW = 0.031 * 0.94 * udtWeatherNow.dblHumidity / 100 * Exp(udtWeatherNow.dblTemperature / 15) * Exp(-udtLocation.dblHeight / 8200)
    End Select
End Sub
Friend Function BSky(dblFix As Double, dblAlt As Double, dblAz As Double, intMode As Integer) As Double
    Dim udtMoon As HBody 'Moon's position
    Dim udtSun As HBody 'Sun's position
    'Components of sky brightness
    Dim dblBNight As Double 'Night
    Dim dblBMoon As Double 'From the Moon
    Dim dblBTwi As Double 'Twighlight according to Schaefer
    Dim dblBTwi2 As Double 'Twighlight by interpolating Koomen's data
    Dim dblBDay As Double 'From the Sun
    Dim dblK As Double 'Overall extinction coefficient
    Dim dblKA As Double
    Dim dblKR As Double
    Dim dblKOz As Double
    Dim dblKW As Double
    Dim dblX As Double 'Overall path length
    Dim dblXM As Double 'Path length in the direction of the Moon
    Dim dblXS As Double 'Path length in the direction of the Sun
    Dim dblZ As Double 'Zenith angle (90 - dblAlt)
    Dim dblZM As Double 'Zenith angle of Moon
    Dim dblZS As Double 'Zenith angle of Sun
    Dim dblRhoM As Double 'Angular separation and function of angular separation to Moon
    Dim dblRhoS As Double 'Angular separation and function of angular separation to Sun
    Dim dblRhoS2 As Double
    Dim dblC4S As Double
    Dim intI As Integer
    Dim dblDAZ As Double
    Dim varTemp As Variant
'    Dim cMoon As New clsMoon
'    Dim cSun As New clsSun
    Dim cHBody As New clsHBody
    
    Call cHBody.TrueHBody("Moon", dblFix, udtMoon)
    Call cHBody.TrueHBody("Sun", dblFix, udtSun)
    'For debugging
    'dblAlt = 0
    'dblAz = 0
    'udtSun.AltRefr = -3
    'udtSun.AzTopo = 0
    'udtSun.RefrValid = True
    Call GetK(dblFix, intMode, udtSun.RATopo, udtSun.HourAngTopo, dblKA, dblKOz, dblKR, dblKW)
    dblK = dblKA + dblKR + dblKOz + dblKW
    dblZ = 90 - dblAlt
    'Find angular separation from the Moon
    dblRhoM = sinD(udtMoon.AltRefr) * sinD(dblAlt) + cosD(udtMoon.AltRefr) * cosD(dblAlt) * cosD(udtMoon.AzTopo - dblAz)
    If dblRhoM > 0.999995 Then
        dblRhoM = Sqr(Square(dblAz - udtMoon.AzTopo) * cosD((udtMoon.AltRefr + dblAlt) / 2) + Square(udtMoon.AltRefr - dblAlt))
    ElseIf dblRhoM <= -1 Then
        dblRhoM = 180
    Else
        dblRhoM = acosD(dblRhoM)
    End If
    If dblRhoM < udtMoon.Phase * 2 * udtMoon.Radius * 0.97 Then dblRhoM = udtMoon.Phase * 2 * udtMoon.Radius * 0.97 'An approximation for the smallest effective distance
    dblRhoS = sinD(udtSun.AltRefr) * sinD(dblAlt) + cosD(udtSun.AltRefr) * cosD(dblAlt) * cosD(udtSun.AzTopo - dblAz)
    If dblRhoS > 0.999995 Then
        dblRhoS = Sqr(Square(dblAz - udtSun.AzTopo) * cosD((udtSun.AltRefr + dblAlt) / 2) + Square(udtSun.AltRefr - dblAlt))
    ElseIf dblRhoS <= -1 Then
        dblRhoS = 180
    Else
        dblRhoS = acosD(dblRhoS)
    End If
    If dblRhoS < 2 * udtSun.Radius * 0.97 Then dblRhoS = 2 * udtSun.Radius * 0.97
    Select Case intMode
    Case 0
        dblX = 1 / (cosD(dblZ) + 0.025 * Exp(-11 * cosD(dblZ)))
        If udtMoon.RefrValid And udtMoon.AltRefr > 0 Then
            dblXM = 1 / (cosD(90 - udtMoon.AltRefr) + 0.025 * Exp(-11 * cosD(90 - udtMoon.AltRefr)))
        Else
            dblXM = 40
        End If
        If udtSun.RefrValid And udtSun.AltRefr > 0 Then
            dblXS = 1 / (cosD(90 - udtSun.AltRefr) + 0.025 * Exp(-11 * cosD(90 - udtSun.AltRefr)))
        Else
            dblXS = 40
        End If
        dblBNight = 90.09 * (1 + 0.3 * Cos(6.283 * (dblFix / 365.242 - 1991) / 11.2)) * (0.4 + 0.6 / Sqr(1 - 0.96 * Square(sinD(dblZ)))) * 10 ^ (-0.4 * dblK * dblX)
        'Find angular separation from the Moon
        dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoM))) + 10 ^ (6.15 - dblRhoM / 40) + 62000000# / Square(dblRhoM)
        dblBMoon = dblRhoM * 10 ^ (-0.4 * (cHBody.Magnitude("Moon", udtMoon) + 16.57 + dblK * dblXM)) * (1 - 10 ^ (-0.4 * dblK * dblX))
        dblBTwi = 10 ^ ((dblRhoS / 90) - 1.1)
        If dblBTwi < 1 Then dblBTwi = 1
        dblBTwi = dblBTwi * 10 ^ (8.45 + 0.4 * udtSun.AltTopo) * (1 - 10 ^ (-0.4 * dblK * dblX))
        dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoS))) + 10 ^ (6.15 - (dblRhoS / 40)) + 62000000# / Square(dblRhoS)
        dblBDay = 11700 * dblRhoS * 10 ^ (-0.4 * dblK * dblXS) * (1 - 10 ^ (-0.4 * dblK * dblX))
        If dblBTwi < dblBDay Then
            BSky = dblBTwi
        Else
            BSky = dblBDay
        End If
        BSky = (BSky + dblBNight + dblBMoon) * 0.00000318
    Case 1
        dblX = 1 / (cosD(dblZ) + 0.025 * Exp(-11 * cosD(dblZ)))
        If Not udtMoon.RefrValid Or udtMoon.AltRefr < 0 Then
            dblXM = 40
        Else
            dblZM = 90 - udtMoon.AltRefr
            dblXM = 1 / (cosD(dblZM) + 0.025 * Exp(-11 * cosD(dblZM)))
        End If
        If Not udtSun.RefrValid Or udtSun.AltRefr < 0 Then
            dblXS = 40
        Else
            dblZS = 90 - udtSun.AltRefr
            dblXS = 1 / (cosD(dblZS) + 0.025 * Exp(-11 * cosD(dblZS)))
        End If
        dblBNight = 0.0000000000001 * (1 + 0.3 * Cos(6.283 * (dblFix / 365.242 - 1991) / 11.2)) * (0.4 + 0.6 / Sqr(1 - 0.96 * Square(sinD(dblZ)))) * 10 ^ (-0.4 * dblK * dblX)
        dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoM))) + 10 ^ (6.15 - dblRhoM / 40) + 62000000# / Square(dblRhoM)
        dblC4S = 10 ^ (-0.4 * dblK / dblXM)
        dblBMoon = 10 ^ (-0.4 * (cHBody.Magnitude("Moon", udtMoon) + 11.05 + 43.27)) * (1 - 10 ^ (-0.4 * dblK * dblX)) * (dblRhoM * dblC4S + 440000 * (1 - dblC4S))
        dblBTwi = 10 ^ (-0.4 * (-26.74 + 11.05 + 32.5 - udtSun.AltTopo - dblZ / 360 / dblK)) * 100 / dblRhoS * (1 - 10 ^ (-0.4 * dblK))
        dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoS))) + 10 ^ (6.15 - (dblRhoS / 40)) + 62000000# / Square(dblRhoS)
        dblC4S = 10 ^ (-0.4 * dblK / dblXS)
        dblBDay = 10 ^ (-0.4 * (-26.74 + 11.05 + 43.27)) * (1 - 10 ^ (-0.4 * dblK * dblX)) * (dblRhoS * dblC4S + 440000 * (1 - dblC4S))
        If dblBTwi > dblBDay Then
            BSky = dblBDay + dblBNight
        Else
            BSky = dblBTwi + dblBNight
        End If
        If udtMoon.RefrValid And udtMoon.AltRefr >= 0 Then
            BSky = BSky + dblBMoon
        End If
        BSky = BSky / 1.11E-15 * 0.00000318
    Case 2 'Hoffman & Kaatz in press when Sun's altitude between -15 and 5 and more than a degree from the Sun, Schaefer 1993 otherwise
        If dblZ > 90 Then
            dblX = 40
        Else
            dblX = Abs(1 / (cosD(dblZ) + 0.025 * Exp(-11 * cosD(dblZ))))
            If dblX > 40 Then dblX = 40
        End If
        dblRhoS2 = dblRhoS
        If udtSun.AltTopo > -19.5 And udtSun.AltTopo < 8 Then 'And dblRhoS2 > 1 Then
            If Not blnBTwi Then
                Open "BTwi.txt" For Input Access Read As 1
                varTemp = GetValue(1, "#", "Long", 0, 256)
                If IsNull(varTemp) Then
                    MsgBox "Error in BTwi file"
                    End
                End If
                dblBTwiData.intNHS = varTemp
                varTemp = GetValue(1, "#", "Long", 0, 256)
                If IsNull(varTemp) Then
                    MsgBox "Error in BTwi file"
                    End
                End If
                dblBTwiData.intNAlt = varTemp
                varTemp = GetValue(1, "#", "Long", 0, 256)
                If IsNull(varTemp) Then
                    MsgBox "Error in BTwi file"
                    End
                End If
                dblBTwiData.intNDAZ = varTemp
                ReDim dblBTwiData.HS(dblBTwiData.intNHS - 1)
                ReDim dblBTwiData.H(dblBTwiData.intNAlt - 1)
                ReDim dblBTwiData.DAZ(dblBTwiData.intNDAZ - 1)
                ReDim dblBTwiData.LogB(dblBTwiData.intNHS * dblBTwiData.intNAlt * dblBTwiData.intNDAZ - 1)
                ReDim dblBTwiData.Y2(dblBTwiData.intNHS * dblBTwiData.intNAlt * dblBTwiData.intNDAZ - 1)
                For intI = 0 To dblBTwiData.intNHS - 1
                    varTemp = GetValue(1, "#", "Double", -90, 90)
                    If IsNull(varTemp) Then
                        MsgBox "Error in BTwi file"
                        End
                    End If
                    dblBTwiData.HS(intI) = varTemp
                Next
                For intI = 0 To dblBTwiData.intNAlt - 1
                    varTemp = GetValue(1, "#", "Double", -90, 90)
                    If IsNull(varTemp) Then
                        MsgBox "Error in BTwi file"
                        End
                    End If
                    dblBTwiData.H(intI) = varTemp
                Next
                For intI = 0 To dblBTwiData.intNDAZ - 1
                    varTemp = GetValue(1, "#", "Double", 0, 180)
                    If IsNull(varTemp) Then
                        MsgBox "Error in BTwi file"
                        End
                    End If
                    dblBTwiData.DAZ(intI) = varTemp
                Next
                For intI = 0 To dblBTwiData.intNHS * dblBTwiData.intNAlt * dblBTwiData.intNDAZ - 1
                    varTemp = GetValue(1, "#", "Double", -4, 5)
                    If IsNull(varTemp) Then
                        MsgBox "Error in BTwi file"
                        End
                    End If
                    dblBTwiData.LogB(intI) = varTemp
                Next
                Close 1
                Call splie2(dblBTwiData.HS, dblBTwiData.LogB, dblBTwiData.Y2, dblBTwiData.intNHS, dblBTwiData.intNAlt * dblBTwiData.intNDAZ)
                blnBTwi = True
            End If
            dblDAZ = FracCirc(udtSun.AzTopo - dblAz)
            If dblDAZ > 180 Then dblDAZ = 360 - dblDAZ
            dblBTwi2 = 10 ^ splin3(dblBTwiData.DAZ, dblBTwiData.H, dblBTwiData.HS, dblBTwiData.LogB, dblBTwiData.Y2, dblBTwiData.intNDAZ, dblBTwiData.intNAlt, dblBTwiData.intNHS, dblDAZ, dblAlt, udtSun.AltTopo)
            'Correct for altitude
            dblBTwi2 = dblBTwi2 * (0.55 + 0.45 * Exp(-udtLocation.dblHeight / 2770 * Exp(-dblAlt / 25)))
        End If
        If udtMoon.RefrValid And udtMoon.AltRefr > 0 Then
            dblXM = 1 / (cosD(90 - udtMoon.AltRefr) + 0.025 * Exp(-11 * cosD(90 - udtMoon.AltRefr)))
            If dblXM > 40 Then dblXM = 40
        Else
            dblXM = 40
        End If
        If udtSun.RefrValid And udtSun.AltRefr > 0 Then
            dblXS = 1 / (cosD(90 - udtSun.AltRefr) + 0.025 * Exp(-11 * cosD(90 - udtSun.AltRefr)))
            If dblXM > 40 Then dblXM = 40
        Else
            dblXS = 40
        End If
        dblBNight = 0.0002865 * (1 + 0.3 * Cos(TwoPi * (dblFix / 365.242 - 2000.21) / 11.2)) * (0.4 + 0.6 / Sqr(1 - 0.96 * Square(sinD(dblZ)))) * 10 ^ (-0.4 * dblK * dblX)
        'Find angular separation from the Moon
        dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoM))) + 10 ^ (6.15 - dblRhoM / 40) + 62000000# / Square(dblRhoM)
        dblBMoon = 0.00000318 * dblRhoM * 10 ^ (-0.4 * (cHBody.Magnitude("Moon", udtMoon) + 16.57 + dblK * dblXM)) * (1 - 10 ^ (-0.4 * dblK * dblX))
        dblBTwi = 10 ^ ((dblRhoS / 90) - 1.1)
        If dblBTwi < 1 Then dblBTwi = 1
        dblBTwi = 0.00000318 * dblBTwi * 10 ^ (8.45 + 0.4 * udtSun.AltTopo) * (1 - 10 ^ (-0.4 * dblK * dblX))
        dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(dblRhoS))) + 10 ^ (6.15 - (dblRhoS / 40)) + 62000000# / Square(dblRhoS)
        dblBDay = 0.372 * dblRhoS * 10 ^ (-0.4 * dblK * dblXS) * (1 - 10 ^ (-0.4 * dblK * dblX))
        If dblBTwi < dblBDay Then
            BSky = dblBTwi
        Else
            BSky = dblBDay
        End If
        BSky = (BSky + dblBNight + dblBMoon)
        If udtSun.AltTopo >= -13.5 And udtSun.AltTopo <= 4 Then 'And dblRhoS2 >= 1 Then
            BSky = dblBTwi2
        ElseIf udtSun.AltTopo < -13.5 And udtSun.AltTopo > -19.5 Then 'And dblRhoS2 >= 1 Then
            BSky = BSky * (-13.5 - udtSun.AltTopo) / 6 + dblBTwi2 * (19.5 + udtSun.AltTopo) / 6
        ElseIf udtSun.AltTopo < 8 And udtSun.AltTopo > 4 Then ' And dblRhoS2 >= 1 Then
            BSky = BSky * (udtSun.AltTopo - 4) / 4 + dblBTwi2 * (8 - udtSun.AltTopo) / 4
        End If
        BSky = BSky + dblBMoon
    End Select
End Function
Friend Sub spline(dblX() As Double, dblY() As Double, dblY2() As Double, intN As Integer)
    'For a set of intN points (dblX, dblY) compute the 2nd derivatives (dblY2) for a natural spline
    Dim dblU() As Double 'Work space
    Dim intI As Integer 'Loop counter
    Dim dblSig As Double
    Dim dblP As Double
    
    ReDim dblU(intN)
    dblY2(0) = 0
    dblU(0) = 0
    For intI = 1 To intN - 2
        dblSig = (dblX(intI) - dblX(intI - 1)) / (dblX(intI + 1) - dblX(intI - 1))
        dblP = dblSig * dblY2(intI - 1) + 2
        dblY2(intI) = (dblSig - 1) / dblP
        dblU(intI) = (dblY(intI + 1) - dblY(intI)) / (dblX(intI + 1) - dblX(intI)) - (dblY(intI) - dblY(intI - 1)) / (dblX(intI) - dblX(intI - 1))
        dblU(intI) = (6 * dblU(intI) / (dblX(intI + 1) - dblX(intI - 1)) - dblSig * dblU(intI - 1)) / dblP
    Next
    If intN > 0 Then dblY2(intN - 1) = 0
    For intI = intN - 2 To 0 Step -1
        dblY2(intI) = dblY2(intI) * dblY2(intI + 1) + dblU(intI)
    Next
End Sub
Friend Function splint(dblX() As Double, dblY() As Double, dblY2() As Double, intN As Integer, dblXGiven As Double) As Double
    Dim intKLo As Integer
    Dim intKHi As Integer
    Dim intK As Integer
    Dim dblH As Double
    Dim dblB As Double
    Dim dblA As Double
    
    'Find the points that bracket dblXGiven
    intKLo = 0
    intKHi = intN - 1
    While intKHi - intKLo > 1
        intK = Int((intKHi + intKLo) / 2)
        If dblX(intK) > dblXGiven Then
            intKHi = intK
        Else
            intKLo = intK
        End If
    Wend
    If intKHi < 0 Or intKLo < 0 Then
        MsgBox ("Bad intKHi and intKLo array in SPLINT.")
        splint = dblX(0)
    Else
        dblH = dblX(intKHi) - dblX(intKLo)
        If dblH = 0 Then
            MsgBox ("Bad dblX array in SPLINT.")
            splint = dblX(intKHi)
        Else
            dblA = (dblX(intKHi) - dblXGiven) / dblH
            dblB = (dblXGiven - dblX(intKLo)) / dblH
            splint = dblA * dblY(intKLo) + dblB * dblY(intKHi) + ((dblA * dblA * dblA - dblA) * dblY2(intKLo) + (dblB * dblB * dblB - dblB) * dblY2(intKHi)) * dblH * dblH / 6
        End If
    End If
End Function
Private Sub splie2(dblX() As Double, dblY() As Double, dblY2() As Double, intNPoints As Integer, intNVecs As Integer)
    Dim intI As Integer
    Dim intJ As Integer
    Dim intBase As Integer
    Dim dblYLoc() As Double
    Dim dblY2Loc() As Double
    
    ReDim dblYLoc(intNPoints)
    ReDim dblY2Loc(intNPoints)
    intBase = 0
    For intI = 0 To intNVecs - 1
        For intJ = 0 To intNPoints - 1
            dblYLoc(intJ) = dblY(intBase + intJ)
        Next
        Call spline(dblX, dblYLoc, dblY2Loc, intNPoints)
        For intJ = 0 To intNPoints - 1
            dblY2(intBase + intJ) = dblY2Loc(intJ)
        Next
        intBase = intBase + intNPoints
    Next
End Sub
Private Function splin2(dblX1() As Double, dblX2() As Double, dblY() As Double, dblY2() As Double, intN1 As Integer, intN2 As Integer, dblXGiven1 As Double, dblXGiven2 As Double) As Double
    Dim intI As Integer
    Dim intJ As Integer
    Dim intBase As Integer
    Dim dblXTemp() As Double
    Dim dblYTemp() As Double
    Dim dblYYTemp() As Double
    Dim dblY2Temp() As Double
    Dim dblYY2Temp() As Double
    
    ReDim dblXTemp(intN2 - 1)
    ReDim dblYTemp(intN2 - 1)
    ReDim dblYYTemp(intN1 - 1)
    ReDim dblY2Temp(intN2 - 1)
    ReDim dblYY2Temp(intN1 - 1)
    intBase = intN2 - 1
    For intJ = 0 To intN2 - 1
        dblXTemp(intJ) = dblX2(intBase - intJ)
    Next
    For intI = 0 To intN1 - 1
        For intJ = 0 To intN2 - 1
            dblYTemp(intJ) = dblY(intBase - intJ)
            dblY2Temp(intJ) = dblY2(intBase - intJ)
        Next
        intBase = intBase + intN2
        dblYYTemp(intI) = splint(dblXTemp, dblYTemp, dblY2Temp, intN2, dblXGiven2)
    Next
    Call spline(dblX1, dblYYTemp, dblYY2Temp, intN1)
    splin2 = splint(dblX1, dblYYTemp, dblYY2Temp, intN1, dblXGiven1)
End Function
Private Function splin3(dblX1() As Double, dblX2() As Double, dblX3() As Double, dblY() As Double, dblY2() As Double, intN1 As Integer, intN2 As Integer, intN3 As Integer, dblXGiven1 As Double, dblXGiven2 As Double, dblXGiven3 As Double) As Double
    Dim intI As Integer
    Dim intJ As Integer
    Dim intBase As Integer
    Dim dblYTemp() As Double
    Dim dblYYTemp() As Double
    Dim dblY2Temp() As Double
    Dim dblYY2Temp() As Double
    
    ReDim dblYTemp(intN2 * intN3 - 1)
    ReDim dblYYTemp(intN1 - 1)
    ReDim dblY2Temp(intN2 * intN3 - 1)
    ReDim dblYY2Temp(intN1 - 1)
    intBase = 0
    For intI = 0 To intN1 - 1
        For intJ = 0 To intN2 * intN3 - 1
            dblYTemp(intJ) = dblY(intBase + intJ)
            dblY2Temp(intJ) = dblY2(intBase + intJ)
        Next
        intBase = intBase + intN2 * intN3
        dblYYTemp(intI) = splin2(dblX2, dblX3, dblYTemp, dblY2Temp, intN2, intN3, dblXGiven2, dblXGiven3)
    Next
    Call spline(dblX1, dblYYTemp, dblYY2Temp, intN1)
    splin3 = splint(dblX1, dblYYTemp, dblYY2Temp, intN1, dblXGiven1)
End Function
Function ProbVis(dblArea As Double, dblPerimeter As Double, dblBSky As Double, dblAppMag As Double, dblKA As Double, dblXA As Double) As Double
'Calculate the probability of visibility from photometric paramters
'Take into account the shape of the object
'Input parameters    :
'       dblArea         solid angle of object (square degrees)
'       dblperimeter    angular length of the perimeter (degrees)
'       dblBSky         Background brightness (Cd/m^2)
'       dblAppMag       Apparent stellar magnitude of the object
'       dblKA           absorption constant due to aerosols
'       dblXA           aerosol path length (atmospheres)

    Dim dblUsefulPerimeter As Double 'Effective perimeter seen by the eye (degrees)
    Dim dblUsefulWidth As Double 'Width of the edge seen by the eye (degrees)
    Dim dblUsefulArea As Double 'Effective area used by the eye (square degrees)
    Dim dblIlluminance As Double 'Illuminance of object (foot candles)
    Dim dblUsefulIlluminance As Double 'Illuminance defining the edge as seen by the eye (foot candles)
    Dim dblUsefulRatio As Double 'Ratio of useful area to area
    Dim dblBright As Double 'Brightness of object (cd/m^2)
    Dim dblA As Double 'Coefficients for threshold contrast calculation
    Dim dblB As Double
    Dim dblC As Double 'As above and Contrast
    Dim dblCT As Double 'Threshold contrast
    Dim dblC2 As Double 'Upper limit of contrast to be considered in integration
    Dim intI As Integer 'Loop counters for integration
    Dim intJ As Integer
    Dim dblP As Double 'Probability scaling factor
    Dim dblP2 As Double 'Probability scaling factor
    Dim dblP3 As Double 'Probability scaling factor
    Dim dblN As Double 'Normalization constants for integration
    Dim dblN2 As Double
    Dim dblLogB As Double 'log10 of backround brightness
    Dim dblMaxC As Double 'Highest contrast considered
    Dim dblStep As Double 'Integration intervals
    Dim dblStep2 As Double
    Dim dblSig As Double
    Dim dblTemp As Double
    Const KAFactor = 1.16 'Was 2.48
    Const SCurve = 0.13 ' 0.32 Adjustments to fit straight line
    Const Height = 1.57 '1.201
    Const SCurveP = 0.13 ' 0.32 Adjustments to fit straight line
    Const HeightP = 1.57 '1.201
    Const MinPerimeter = 0.21
    'Was 0.092417
    Dim SigFactor As Double
    Dim CTFactor As Double
    
    SigFactor = 1 / Sqr(SCurve * Height)
    CTFactor = Sqr(SCurve / Height)
    dblLogB = Log10(dblBSky)
    'The eye will always react to a minimum perimeter
    If dblPerimeter < MinPerimeter Then
        dblUsefulPerimeter = MinPerimeter
        SigFactor = 1 / Sqr(SCurveP * HeightP)
        CTFactor = Sqr(SCurveP / HeightP)
    ElseIf dblPerimeter > 18 Then
        dblUsefulPerimeter = 18
    Else
        dblUsefulPerimeter = dblPerimeter
    End If
    dblUsefulWidth = UsefulWidth(dblBSky)
    'Calculate the useful area
    dblUsefulArea = dblUsefulPerimeter * dblUsefulWidth
    'Modification to prevent the useful area being larger than the area unles it is small
    'This appeared to improve things but the old method produced a straight line log-log graph
    'i.e. may be more correct but requires another parameter tweaked
    'If dblUsefulArea > dblArea Then
    '    dblUsefulArea = dblArea
    'End If
    'The lower limit on the area has no effect on the Moon
    'If dblUsefulArea < MinPerimeter * (dblUsefulWidth) Then dblUsefulArea = MinPerimeter * (dblUsefulWidth)
    'End modification 'min length was 0.092417
    'The useful ratio is used to determine the brightness of the object
    If dblArea > 0 And dblUsefulArea < dblArea Then
        dblUsefulRatio = dblUsefulArea / dblArea
    Else
        dblUsefulRatio = 1
    End If
    'Calculate the illuminance, the illuminance used by the eye and the brighness of the object
    dblIlluminance = (-dblAppMag - 16.57) / 2.5
    If dblIlluminance > 80 Then
        dblIlluminance = 1E+80
    Else
        dblIlluminance = 10 ^ (dblIlluminance)
    End If
    dblUsefulIlluminance = dblIlluminance * dblUsefulRatio
    dblBright = dblUsefulIlluminance / dblUsefulArea / 0.0000287
    'Find the threshold contrast
    dblA = -10 ^ (-0.6175 * dblLogB - 2.2352) + 0.000252
    If -0.458 * dblLogB - 0.0582 < 0.0935 * Square(dblLogB) - 0.6335 * dblLogB - 0.3052 And dblLogB < 0 Then
        dblB = -0.458 * dblLogB - 0.0582
    Else
        dblB = 0.0935 * Square(dblLogB) - 0.6335 * dblLogB - 0.3052
    End If
    dblB = 10 ^ dblB
    If -0.6093 * dblLogB - 0.2162 < 0.083338 * Square(dblLogB) - 0.5919 * dblLogB - 0.61167 And dblLogB < 0 Then
        dblC = -0.6093 * dblLogB - 0.2162
    Else
        dblC = 0.083338 * Square(dblLogB) - 0.5919 * dblLogB - 0.61167
    End If
    dblC = 10 ^ dblC
    dblCT = dblA * Square(dblUsefulPerimeter) + dblB * dblUsefulPerimeter + dblC
    dblCT = dblCT * CTFactor
    'Calculate the actual contrast
    dblC = dblBright / dblBSky
    'Probability functions
    'Determine the maximum possible contrast, when KA = 0
    dblTemp = (-16.57 - dblAppMag + dblKA * dblXA) / 2.5
    If dblTemp > 80 Then
        dblMaxC = dblUsefulRatio * 1E+80 / 0.0000287 / dblUsefulArea / dblBSky
    Else
        dblMaxC = dblUsefulRatio * 10 ^ (dblTemp) / 0.0000287 / dblUsefulArea / dblBSky
    End If
    'Numercally integrate the probability function with respect to KA
    dblStep = dblMaxC / intNIntegrals
    dblSig = (0.02445 * Log(dblUsefulPerimeter) + 0.379) * SigFactor
    'Integrate for dblC
    dblP2 = 0
    If dblStep > 0 Then
    For intI = 1 To intNIntegrals - 1
        dblTemp = (-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA
        If dblTemp > 0 Then dblP2 = dblP2 + Exp(-2.188 * Square(Log(dblTemp)) - Square(Log((dblC + intI * dblStep) / dblCT) / dblSig) / 2)
    Next
    End If
    dblP = dblP2
    'Increase the upper C limit until its integral becomes small compared with that for dblC
    dblC2 = dblC
    While dblP2 * intNIntegrals > dblP
        dblC2 = dblC2 * 1.4142135
        dblP2 = 0
        For intI = 1 To intNIntegrals - 1
            dblP2 = dblP2 + Exp(-2.188 * Square(Log((-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA)) - Square(Log((dblC2 + intI * dblStep) / dblCT) / (dblSig)) / 2)
        Next
    Wend
    'Integrate the function
    dblP2 = 0
    dblP3 = 0
    dblStep2 = (dblC2 - dblC) / intNIntegrals
    If dblStep2 > 0 Then
    For intI = 1 To intNIntegrals - 1
        dblTemp = (-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA
        If dblTemp > 0 Then
            dblTemp = 2.188 * Square(Log(dblTemp))
            dblP3 = Exp(-dblTemp - Square(Log((dblC + intI * dblStep) / dblCT) / (dblSig)) / 2) / 2
            For intJ = 1 To intNIntegrals
                dblP3 = dblP3 + Exp(-dblTemp - Square(Log((dblC + intJ * dblStep2 + intI * dblStep) / dblCT) / (dblSig)) / 2)
                'Debug.Print Exp(-Square(Log((-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA)) / 0.8281 / 2 - Square(Log((dblC + intJ * dblStep2 + intI * dblStep) / dblCT) / (dblSig)) / 2)
            Next
        Else
            dblP3 = 0
        End If
        'Debug.Print dblP3
        dblP2 = dblP2 + dblP3
    Next
    End If
    'Debug.Print
    dblP2 = dblP2 * dblStep * dblStep2
    dblStep2 = dblC / intNIntegrals
    dblP = 0
    If dblStep > 0 Then
    For intI = 1 To intNIntegrals - 1
        dblP3 = 0
        dblTemp = (-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA
        If dblTemp > 0 Then
            dblTemp = 2.188 * Square(Log(dblTemp))
            For intJ = 0 To intNIntegrals - 1
                dblP3 = dblP3 + Exp(-dblTemp - Square(Log((intJ * dblStep2 + intI * dblStep) / dblCT) / dblSig) / 2)
                'Debug.Print Exp(-Square(Log((-16.57 - 2.5 * Log10(0.0000287 * dblUsefulArea * intI * dblStep * dblBSky / dblUsefulRatio) - dblAppMag + dblKA * dblXA) / KAFactor / dblXA)) / 0.8281 / 2 - Square(Log((intJ * dblStep2 + intI * dblStep) / dblCT) / (dblSig)) / 2)
            Next
            dblP3 = dblP3 + Exp(-dblTemp - Square(Log((intNIntegrals * dblStep2 + intI * dblStep) / dblCT) / dblSig) / 2) / 2
        End If
        'Debug.Print dblP3
        dblP = dblP + dblP3
    Next
    End If
    dblP = dblP * dblStep * dblStep2
    If dblP + dblP2 > 0 Then dblP = dblP / (dblP + dblP2)
    ProbVis = dblP
'    End If
End Function
Function UsefulWidth(dblBSky As Double) As Double
    'The effective width is related to the eye's resolution as a function of backgroud brightness
    If dblBSky > 10 ^ (-0.3283) Then
        UsefulWidth = 0.01111111 * 10 ^ (8.28 * (dblBSky * 3145.6) ^ -0.29)
    Else
        UsefulWidth = 0.105 * 10 ^ (0.3 * (dblBSky * 3145.6) ^ -0.29)
        If UsefulWidth > 0.25 Then UsefulWidth = 0.25
    End If
End Function
Friend Function SeaLevelHorizon(udtWeartherNow As WeatherNow) As Double
'Calculate apparent angle of dip to the sea
    Dim dblPSL As Double
    Dim dblTSL As Double
    
    If udtLocation.dblHeight > 0 Then
        dblPSL = udtWeatherNow.dblPressure * Exp(udtLocation.dblHeight / 7143) / 1013.25
        dblTSL = (udtWeatherNow.dblTemperature + 0.005688 * udtLocation.dblHeight + 273.15) / 273.15
        SeaLevelHorizon = -0.626 - 1.11 * Sqr(udtLocation.dblHeight / 1000) * dblPSL / dblTSL
    Else
        SeaLevelHorizon = 0
    End If
End Function

